#!/bin/sh

# HACKS!!!!
#
# Remove VALAFLAGS="--target-glib=2.50"

apt-get update
apt-get upgrade -y
apt-get install -y \
	appstream-util \
	autoconf \
	autoconf-archive \
	autogen \
	automake \
	autopoint \
	bison \
	build-essential \
	check \
	curl \
	dbus \
	dbus-x11 \
	devscripts \
	emacs \
	emacs-goodies-el \
	flex \
	gcc \
	gettext \
	git \
	gnulib \
	gobject-introspection \
	gperf \
	graphviz \
	help2man \
	imagemagick \
	indent \
	jekyll \
	jq \
	libcairo2-dev \
	libgirepository1.0-dev \
	libglib2.0-dev \
	libgtk-3-dev \
	libreadline-dev \
	libtool \
	libxml2 \
	libxml2-dev \
	libxml2-utils \
	make \
	nettle-dev \
	org-mode \
	pkg-config \
	r-cran-ggplot2 \
	texinfo \
	valac \
	valgrind \
	xsltproc

export $(dbus-launch)

SOURCE_TOP=$(pwd)
CLEAN_TOP=$(cd .. && pwd)/clean
mkdir -p ../clean
chmod -R ugo+w ../clean || exit 1
rm -rf ../clean || exit 1
mkdir -p ../clean || exit 1
rm -rf $SOURCE_TOP/dist || exit 1

cd $CLEAN_TOP
git clone $SOURCE_TOP . || exit 1

cd $SOURCE_TOP || exit 1
rm -rf ../tarot-*.tar.gz ../tarot-*/ tarot-*.tar.gz
sh autogen.sh VALAFLAGS="--target-glib=2.50" || exit 1
./configure \
    CFLAGS="-Wall -Wextra -Werror -g $CFLAGS" \
    VALAFLAGS="--target-glib=2.50" \
    --prefix="$SOURCE_TOP/dist" \
    --enable-valgrind=yes \
    --enable-silent-rules=yes \
    || (cat config.log ; exit 1) \
    || exit 1
(cd lib && make -j 16 CFLAGS="-Wno-error -g $CFLAGS") || exit 1
make -j 16 safe || exit 1
make -j 16 distcheck || exit 1
make -j 8 check || exit 1
make -j 16 install install-html || exit 1
make indent || exit 1
rm -rf ../../tarot-*.tar.gz ../../tarot-*/
cp tarot-*.tar.gz ../../ || exit 1
cd ../../ || exit 1
tar xf tarot-*.tar.gz || exit 1
cd tarot-* || exit 1
DIST_TOP=$(pwd)
cd $CLEAN_TOP || exit 1
OK="yes"
for source in $(find . -type f)
do
    case $source in
	*.po | *.pot)
	    echo "Ignoring PO file $source, as it has a date in it.";;
	*.md)
	    echo "Ignoring Markdown file $source, as it may be non-reproducibly generated by org-mode.";;
	*/.git/*);;
	*)
	    if diff $source $DIST_TOP/$source
	    then
		echo "$source: OK"
	    else
		echo "ERROR: $source has been modified since check-in, or ignored from the distribution.  Is it automatically generated?  Did you forget to indent it?  Did you forget to include it as distributed in Automake?"
		OK="no"
	    fi;;
	esac
done;
cd $DIST_TOP || exit 1
./configure && make maintainer-clean || exit 1
echo "Checking what files are maintainer-clean..."
for source in $(find . -type f)
do
    case $source in
	*.po | *.pot)
	    echo "Ignoring PO file $source, as it has a date in it.";;
	./po/en@quot.header | ./po/en@boldquot.header | ./po/Rules-quot | ./po/remove-potcdate.sin | ./po/boldquot.sed | ./po/quot.sed | ./po/insert-header.sin | ./po/Makefile.in.in)
	    echo "Ignoring Gettext garbage $source.";;
	./libopts/*)
	    echo "Ignoring embedded file $source, as it is not ours.";;
	./.tarball-version)
	    echo "Ignoring .tarball-version.";;
	./lib/* | ./snippet/*)
	    echo "Ignoring $source, some gnulib stuff.";;
	./m4/*)
	    echo "Ignoring autoconf macro $source.";;
	./git-version-gen | ./gitlog-to-changelog)
	    echo "Ignoring helper script $source.";;
	./compile | ./depcomp | ./config.rpath | ./config.sub | ./texinfo.tex | ./ar-lib | ./INSTALL | ./missing | ./test-driver | ./ltmain.sh | ./aclocal.m4 | ./ChangeLog | ./config.h.in | ./config.guess | ./Makefile.in | ./configure | ./install-sh | ./ABOUT-NLS)
	    echo "Ignoring required automake script $source.";;
	*)
	    if diff $source $CLEAN_TOP/$source
	    then
		echo "$source: OK"
	    else
		echo "ERROR: $source is not a maintainer-clean file, but it has been rebuilt or deleted since checkout.  If this file has been generated, add it to MAINTAINERCLEANFILES."
		OK="no"
	    fi;;
    esac
done;
if test "$OK" = "no"
then
    echo "Failure."
    exit 1
fi

## Nice!
cd $SOURCE_TOP || exit 1
mkdir -p $SOURCE_TOP/dist || exit 1
cp $SOURCE_TOP/.version \
   $SOURCE_TOP/dist/version || exit 1
cp tarot-*.tar.gz $SOURCE_TOP/dist/ || exit 1

if test "x$CI_COMMIT_TAG" != "x"
then
    cd $SOURCE_TOP
    TARFILE=$(ls dist/tarot-*.tar.gz)
    mkdir /root/.ssh || exit 1
    echo "$SSH_KEY" > /root/.ssh/id_rsa || exit 1
    echo "$SSH_KEY_PUB" > /root/.ssh/id_rsa.pub || exit 1
    echo "$SSH_KNOWN_HOSTS" > /root/.ssh/known_hosts || exit 1
    chmod go-rwx /root/.ssh/id_rsa || exit 1
    git config --global user.email "vivien@planete-kraus.eu" || exit 1
    git config --global user.name "Gitlab CI for the tarot package" || exit 1
    wget -O /tmp/submit https://play-tarot.frama.io/tarot-releases/submit || exit 1
    chmod ugo+x /tmp/submit || exit 1
    /tmp/submit $TARFILE || exit 1
fi
