#!/bin/sh

tarot --help > /dev/null || exit 1

AT_DOG=$(tarot --new-game -n 5 -c \
	| tarot --deal-to P5=21T,20T,19T,18T,13T,12T,11T,10T,9T,8T,4T,1T,KH,KD,QD \
	| tarot --bid=take \
	| tarot --bid=pass \
	| tarot --bid=pass \
	| tarot --bid=pass \
	| tarot --bid=push \
	| tarot --declare=slam \
	| tarot --call=KC \
	| tarot --dog=15T,KS,EXC)

STATUS=$(echo "$AT_DOG" | tarot --status)
STATUS_IMPUTED=$(echo "$AT_DOG" | tarot --impute -s "Hello :)" | tarot --status)

STATUS_EXPECTED=$(
    echo "╭────────┬────────┬────────┬────────┬────────╮"
    echo "│   P1   │   P2   │   P3   │   P4   │** P5   │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  take  │  pass  │  pass  │  pass  │  push  │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  slam! │  pass  │  pass  │  pass  │  pass  │"
    echo "╰────────┴────────┴────────┴────────┴────────╯"
    echo " - Call: K♣"
    echo " - Dog: K♠, 15★, EXC"
    echo " - Cards of P5: K♥, Q♦, K♦, 1★, 4★, 8★, 9★, 10★, 11★, 12★, 13★, 18★, 19★, 20★, 21★"
    echo "P5 should discard Q♦, and 2 cards among 4★, 8★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★"
	       )
STATUS_IMPUTED_EXPECTED=$(
    echo "╭────────┬────────┬────────┬────────┬────────╮"
    echo "│   P1 * │   P2   │   P3   │   P4   │** P5   │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  take  │  pass  │  pass  │  pass  │  push  │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  slam! │  pass  │  pass  │  pass  │  pass  │"
    echo "╰────────┴────────┴────────┴────────┴────────╯"
    echo " - Call: K♣"
    echo " - Dog: K♠, 15★, EXC"
    echo " - Cards of P1: 3♥, 6♥, 10♥, 2♣, K♣, 2♦, 3♦, 5♦, J♦, 2♠, 6♠, 10♠, 7★, 14★, 16★"
    echo " - Cards of P2: 2♥, 4♥, J♥, C♥, 4♣, 7♣, 9♣, 10♣, 4♦, 6♦, C♦, Q♠, 2★, 5★, 17★"
    echo " - Cards of P3: A♥, 8♥, 9♥, A♣, 3♣, 6♣, 8♣, 7♦, 10♦, A♠, 3♠, 4♠, 7♠, 8♠, J♠"
    echo " - Cards of P4: 5♥, 7♥, Q♥, 5♣, J♣, C♣, Q♣, A♦, 8♦, 9♦, 5♠, 9♠, C♠, 3★, 6★"
    echo " - Cards of P5: K♥, Q♦, K♦, 1★, 4★, 8★, 9★, 10★, 11★, 12★, 13★, 18★, 19★, 20★, 21★"
    echo "P5 should discard Q♦, and 2 cards among 4★, 8★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★"
		       )

if test "$STATUS" != "$STATUS_EXPECTED"
then
    >&2 echo "FAILED: the status is not correct ('$STATUS' != '$STATUS_EXPECTED')"
    exit 1
fi

if test "$STATUS_IMPUTED" != "$STATUS_IMPUTED_EXPECTED"
then
    >&2 echo "FAILED: the status with imputation is not correct ('$STATUS_IMPUTED' != '$STATUS_IMPUTED_EXPECTED')"
    exit 1
fi

if echo "$AT_DOG" | tarot --discard=9T,8T,4T > /dev/null 2>&1
then
    >&2 echo "FAILED: I was not supposed to be able to discard these"
    exit 1
fi
