#!/bin/sh

tarot --help > /dev/null || exit 1

AT_PLAY=$(tarot --new-game -n 5 -c \
	| tarot --deal-to P1=21T,20T,19T,18T,13T,12T,11T,10T,9T,8T,4T,1T,KH,KD,QD \
	| tarot --bid=take \
	| tarot --bid=pass \
	| tarot --bid=pass \
	| tarot --bid=pass \
	| tarot --bid=pass \
	| tarot --declare=slam \
	| tarot --call=KC \
	| tarot --dog=15T,KS,EXC \
	| tarot --discard=QD,8T,4T)

STATUS=$(echo "$AT_PLAY" | tarot --status)
STATUS_IMPUTED=$(echo "$AT_PLAY" | tarot --impute | tarot --status)

STATUS_EXPECTED=$(
    echo "╭────────┬────────┬────────┬────────┬────────╮"
    echo "│** P1   │   P2   │   P3   │   P4   │   P5   │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  take  │  pass  │  pass  │  pass  │  pass  │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  slam! │  pass  │  pass  │  pass  │  pass  │"
    echo "╰────────┴────────┴────────┴────────┴────────╯"
    echo " - Call: K♣"
    echo " - Dog: K♠, 15★, EXC"
    echo " - Discard: Q♦, 4★, 8★"
    echo "╭────────┬────────┬────────┬────────┬────────╮"
    echo "│** P1   │   P2   │   P3   │   P4   │   P5   │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│        │        │        │        │        │"
    echo "╰────────┴────────┴────────┴────────┴────────╯"
    echo " - Cards of P1: K♥, K♦, K♠, 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★, EXC"
    echo "P1 can show a simple handful with 8 cards among 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★"
    echo "P1 can show a double handful with 10 cards among 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★"
    echo "P1 can show a triple handful with 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★, and 2 cards among 4★, 8★, EXC"
    echo "P1 may play a card among K♥, K♦, K♠, 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★, EXC"
	       )
STATUS_IMPUTED_EXPECTED=$(
    echo "╭────────┬────────┬────────┬────────┬────────╮"
    echo "│** P1   │   P2   │   P3   │   P4   │   P5 * │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  take  │  pass  │  pass  │  pass  │  pass  │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│  slam! │  pass  │  pass  │  pass  │  pass  │"
    echo "╰────────┴────────┴────────┴────────┴────────╯"
    echo " - Call: K♣"
    echo " - Dog: K♠, 15★, EXC"
    echo " - Discard: Q♦, 4★, 8★"
    echo "╭────────┬────────┬────────┬────────┬────────╮"
    echo "│** P1   │   P2   │   P3   │   P4   │   P5 * │"
    echo "├────────┼────────┼────────┼────────┼────────┤"
    echo "│        │        │        │        │        │"
    echo "╰────────┴────────┴────────┴────────┴────────╯"
    echo " - Cards of P1: K♥, K♦, K♠, 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★, EXC"
    echo " - Cards of P2: 3♥, 10♥, J♥, C♥, Q♥, 8♣, J♣, C♣, Q♣, 4♠, 7♠, C♠, 2★, 7★, 17★"
    echo " - Cards of P3: 2♥, 4♥, 7♥, 8♥, 9♥, A♣, 10♣, 2♦, 5♦, 6♦, 10♦, 2♠, 8♠, 9♠, 5★"
    echo " - Cards of P4: A♥, 6♥, 2♣, 4♣, 5♣, 9♣, A♦, 4♦, 7♦, 9♦, C♦, 3♠, 5♠, Q♠, 14★"
    echo " - Cards of P5: 5♥, 3♣, 6♣, 7♣, K♣, 3♦, 8♦, J♦, A♠, 6♠, 10♠, J♠, 3★, 6★, 16★"
    echo "P1 can show a simple handful with 8 cards among 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★"
    echo "P1 can show a double handful with 10 cards among 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★"
    echo "P1 can show a triple handful with 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★, and 2 cards among 4★, 8★, EXC"
    echo "P1 may play a card among K♥, K♦, K♠, 1★, 9★, 10★, 11★, 12★, 13★, 15★, 18★, 19★, 20★, 21★, EXC"
		       )

if test "$STATUS" != "$STATUS_EXPECTED"
then
    >&2 echo "FAILED: the status is not correct ('$STATUS' != '$STATUS_EXPECTED')"
    exit 1
fi

if test "$STATUS_IMPUTED" != "$STATUS_IMPUTED_EXPECTED"
then
    >&2 echo "FAILED: the status with imputation is not correct ('$STATUS_IMPUTED' != '$STATUS_IMPUTED_EXPECTED')"
    exit 1
fi

if echo "$AT_PLAY" | tarot --handful=4T,8T,1T,9T,10T,11T,12T,13T,18T,19T,20T,21T,EXC > /dev/null 2>&1
then
    >&2 echo "FAILED: I was not supposed to be able to show these"
    exit 1
fi
